# File:  DemoProblem.py
#
# Program to interact with the firmware running on the Tiva MCU.  Demonstrates a problem when writing
# a multiple of max_packet_size bytes to EP 0.


import usb.core
import usb.util
import sys


dev = None      # global in function FindDevice


SENSOR_LOAD_FIRMWARE                = 0xA0      # Read/write MCU's address space. wIndex = HI(addr), wValue=LO(addr)
PASCO_USB_SENSOR_GET_FW_GENERATION  = 0xA2
PASCO_USB_SENSOR_UNLOCK_UPGRADE     = 0xA3
GET_FAUX_USB_PRODUCT_ID             = 0xED
PASCO_USB_SENSOR_GET_FIRMWARE_VERSION=0xF2
PASCO_USB_SENSOR_ACCESS_HARDWARE    = 0xFB      # Use with constants below to app to boot loader or vice versa.

# These are passed in with PASCO_USB_SENSOR_ACCESS_HARDWARE, in high byte of setupReq.wIndex.
PASPORT_HW_GO_BOOT_LOADER           = 33    # App: Jump to bootloader.
PASPORT_HW_READ_WRITE_PSOC_FLASH    = 36    # Don't support.
PASPORT_HW_EXECUTE_CODE				= 24    # Bootloader:  Jump to app.  See "drk extension" note below.
# drk extension:
# For PASPORT_HW_EXECUTE_CODE, wValue has special meanings:
#   wValue = 0          Normal.  Instructs bootloader to jump to app.
#   wValue = 0xDEAD     For testing.  Act like jumping to app, but then don't.
#   wValue = 0xABCD     For testing.  Reboot the bootloader.

VENDOR_IN   = 0xC0
VENDOR_OUT  = 0x40

FLASH_UNLOCK_KEY = 3

AppAddr     = 0x6000
RamAddr     = 0x20000000
RamLength   = 0x8000
RamEnd1     = RamAddr + RamLength
RealSP      = 0x20002EF8	
RealPC      = 0x0000BA7B
Neg1        = -1
SP_lt_Ram   = RamAddr - 16
SP_gt_Ram   = RamEnd1 + 16
PC_lt_App   = AppAddr - 16
RamBlockSize= 32
BigRamBlockAddr = RamEnd1 - 4096

## Reminder of how to do a control transfer with this USB library.
##                                       bmRequestType, bmRequest, wValue, wIndex, <OUT: ptr to data> or <IN: num bytes to read>
## OUT:  writecount = dev.ctrl_transfer( VENDOR_OUT,    bmRequest, wValue, wIndex, <ptr to data> )
## IN :  response   = dev.ctrl_transfer( VENDOR_IN,     bmRequest, wValue, wIndex, <num bytes to read> )



def PrintMenu():
	print("\n"
    "0:  Exit program.\n"
    "C:  Read 32 bytes of RAM.     Also C1: (63)    C4 (127)\n"
    "CC: Read 100 bytes of RAM          C2: (64)    C5 (128)\n"
    "CCC:Read 512 bytes of RAM          C3: (65)    C6 (129)\n"
    "D:  Write 32 bytes to RAM     Also D1: (63)    D4 (127)    !: (256)    @: (200)\n"
    "DD: Write 100 bytes to RAM         D2: (64)    D5 (128)                #: (300)\n"
    "DDD:Write 512 bytes to RAM         D3: (65)    D6 (129)                $: (400)\n"
    "V:  Show version\n"
    "Z:  Re-find device\n")
	
    
def GetChoice():
    return input("Enter choice: \n")
    

def FindDevice():
    # find our device
    global dev
    dev = usb.core.find(idVendor=0x0945, idProduct=0x0002)

    # was it found?
    if dev is None:
        raise ValueError('Device not found')
    else:
        print("Device found.")
        # print(dev)
    # set the active configuration. With no arguments, the first
    # configuration will be the active one
    dev.set_configuration()



def ShowVersion():
    try:
        # Read the variable length firmware version struct.
        fwversion = dev.ctrl_transfer( VENDOR_IN, PASCO_USB_SENSOR_GET_FIRMWARE_VERSION, 0, 0, 64 )
        if len(fwversion) >= 3:
            print("Version number ", fwversion[0], ".", fwversion[1])
            verstr=''
            verstr=''.join([chr(x) for x in fwversion])
            print(verstr)
        else:
            print("GET_FIRMWARE_VERSION succeeded, but received ", len(fwversion), " bytes")
    except usb.core.USBError as e:
        print("BAD:  Failed to read version string.")


    



def ShowRamBlock(length):
    addr = BigRamBlockAddr
    rdCount = length
    try:    
        # reply= dev.ctrl_transfer( VENDOR_IN, bmRequest, wValue, wIndex, <num bytes to read> )
        rdData = dev.ctrl_transfer( VENDOR_IN, SENSOR_LOAD_FIRMWARE, addr & 0xffff, addr >> 16, rdCount)
    except usb.core.USBError as e:
        print("BAD:  Failed to read RAM block.")
    else:
        print("Read ", rdCount, "bytes from address $", hex(addr), ": ")
        # print([hex(x) for x in rdData])       # Shows all bytes in single quotes.
        print('[{}]'.format(', '.join(hex(x) for x in rdData)))     # Shows bytes without quotes.

        

def WriteRamBlock(length):
    ##############  WRITE TO RAM near end
    wrData=[]
    for i in range(length):
        wrData.extend([i & 0xFF])
    addr = BigRamBlockAddr
    try:
        #wrCount= dev.ctrl_transfer( VENDOR_OUT,    bmRequest, wValue, wIndex, <ptr to data> )
        wrCount = dev.ctrl_transfer( VENDOR_OUT, SENSOR_LOAD_FIRMWARE, addr & 0xffff, addr >> 16, wrData )
        if (wrCount != len(wrData)):
            print("BAD :  Wrote ", wrCount, "bytes, expected ", len(bytes))
        else: print("GOOD:  Write ", wrCount, "bytes to RAM at addr ", addr, "succeeded.")
    except usb.core.USBError as e:
        print("BAD :  WRITE BLOCK TO RAM failed.")


def ReadBlock(addr, length):
    try:
        #reply= dev.ctrl_transfer( VENDOR_IN, bmRequest, wValue, wIndex, <num bytes to read> )
        data = dev.ctrl_transfer( VENDOR_IN, SENSOR_LOAD_FIRMWARE, addr & 0xffff, addr >> 16, length)
        return data
    except usb.core.USBError:
        print("ReadBlock: Reading ", length, " bytes from ", hex(addr), " failed.")



#############################################################################
#############################################################################
#########################                           #########################
#########################       BEGIN PROGRAM       #########################
#########################                           #########################
#############################################################################
#############################################################################


print("\n\n")
    
FindDevice()
   
while (1):
    PrintMenu()
    c = GetChoice()
    c = c.upper()
    
    if c == '0':
        exit()
        
        
        
        
    elif c == 'C':          # C: Show 32 bytes of RAM
        ShowRamBlock(32)
    elif c == 'CC':         # CC: Show 100 bytes of RAM
        ShowRamBlock(100)
    elif c == 'CCC':        # CCC: Show 512 bytes of RAM
        ShowRamBlock(512)
    elif c == 'C1':         # C1: Show 63 bytes of RAM
        ShowRamBlock(63)
    elif c == 'C2':         # C2: Show 64 bytes of RAM
        ShowRamBlock(64)
    elif c == 'C3':         # C3: Show 65 bytes of RAM
        ShowRamBlock(65)
    elif c == 'C4':         # C4: Show 127 bytes of RAM
        ShowRamBlock(127)
    elif c == 'C5':         # C5: Show 128 bytes of RAM
        ShowRamBlock(128)
    elif c == 'C6':         # C6: Show 129 bytes of RAM
        ShowRamBlock(129)
        
    elif c == 'D':          # D:  Write 32 bytes to RAM
        WriteRamBlock(32)
    elif c == 'DD':         # DD: Write 100 bytees to RAM
        WriteRamBlock(100)
    elif c == 'DDD':        # DDD:Write 512 bytees to RAM
        WriteRamBlock(512)
    elif c == 'D1':         # D1: Write 63 bytes to RAM
        WriteRamBlock(63)
    elif c == 'D2':         # D2: Write 64 bytes to RAM
        WriteRamBlock(64)
    elif c == 'D3':         # D3: Write 65 bytes to RAM
        WriteRamBlock(65)
    elif c == 'D4':         # D4: Write 127 bytees to RAM
        WriteRamBlock(127)
    elif c == 'D5':         # D5: Write 128 bytees to RAM
        WriteRamBlock(128)
    elif c == 'D6':         # D6: Write 129 bytees to RAM
        WriteRamBlock(129)
        
    elif c == '!':         # !: Write 256 bytees to RAM
        WriteRamBlock(256)
    elif c == '@':         # @: Write 200 bytees to RAM
        WriteRamBlock(200)
    elif c == '#':         # #: Write 300 bytees to RAM
        WriteRamBlock(300)
    elif c == '$':         # %: Write 400 bytees to RAM
        WriteRamBlock(400)
        
       
    elif c == 'V':
        ShowVersion()
    elif c == 'Z':
        FindDevice()
        
        