/*
 * LxiBootUsb.c
 *
 * USB functions for bootloader.
 *
 */

#include <stdbool.h>
#include <stdint.h>
#include <string.h>
#include "inc/hw_memmap.h"
#include "inc/hw_types.h"
#include "inc/hw_gpio.h"
#include "inc/hw_sysctl.h"
#include "driverlib/gpio.h"
#include "driverlib/pin_map.h"
#include "driverlib/rom_map.h"
#include "driverlib/sysctl.h"
#include "driverlib/uart.h"
#include "driverlib/usb.h"
#include "usblib/usblib.h"
#include "usblib/device/usbdevice.h"
#include "utils/ustdlib.h"
#include "console.h"
#include <LxiBootUsb.h>

#define TIVA_RAM_BASE       0x20000000      // Beginning of RAM on TIVA, regardless of where apps start usage.
#define TIVA_RAM_LENGTH     0x8000          // 0x8000 = 32KB.
#define TIVA_RAM_END1       (TIVA_RAM_BASE + TIVA_RAM_LENGTH)   // '1' is reminder that it's last addr + 1.

enum {
    SENSOR_LOAD_FIRMWARE            = 0xa0,
    SENSOR_GET_FIRMWARE_VERSION     = 0xf2,
    TEST_SUCCESS                    = 0x88,
    TEST_FAIL                       = 0xFF
};




extern tDeviceInfo  lxiDevInfo;     // In LxiUsbDescriptors.c

static void HandleGetDescriptor		(void *pvInstance, tUSBRequest *psUSBRequest)	{ }
static void HandleRequest			(void *pvInstance, tUSBRequest *psUSBRequest);
static void HandleEP0DataReceived	(void *pvInstance, uint32_t ui32Info);
static void HandleConfigChange		(void *pvInstance, uint32_t ui32Info) 	{ }
static void HandleEP0DataSent		(void *pvInstance, uint32_t ui32Info) 	{ UARTCharPut(UART0_BASE,'>'); }
static void HandleReset				(void *pvInstance)						{ DEBUGOUT(RED"**RESET**"      RESETATTR);  FLUSHUARTTX(); }
static void HandleSuspend			(void *pvInstance)						{ DEBUGOUT(RED"**SUSPEND**"    RESETATTR);	FLUSHUARTTX(); }
static void HandleResume			(void *pvInstance)						{ DEBUGOUT(RED"**RESUME**"     RESETATTR);	FLUSHUARTTX(); }
static void HandleDisconnect		(void *pvInstance)						{ DEBUGOUT(RED"**DISCONNECT**" RESETATTR);  FLUSHUARTTX(); }
static void HandleEndpoints			(void *pvInstance, uint32_t ui32Status) { };
static void HandleDevice			(void *pvInstance, uint32_t ui32Request, void *pvRequestData)	{ }




tUSBRequest setupReply;
tUSBRequest setupReq;

bool        gotUsbRequest  = false;
bool        gotEP0Data     = false;
uint8       xferDataBuf[0x1000];



//*****************************************************************************
const tCustomHandlers lxiUsbHandlers =
{
    HandleGetDescriptor,    // pfnGetDescriptor
    HandleRequest,          // pfnRequestHandler
    0,                      // pfnInterfaceChange
    HandleConfigChange,     // pfnConfigChange
    HandleEP0DataReceived,  // pfnDataReceived
    HandleEP0DataSent,      // pfnDataSent
    HandleReset,            // pfnResetHandler
    HandleSuspend,          // pfnSuspendHandler
    HandleResume,           // pfnResumeHandler
    HandleDisconnect,       // pfnDisconnectHandler
    HandleEndpoints,        // pfnEndpointHandler
    HandleDevice            // pfnDeviceHandler
};






//*****************************************************************************
void LxiUsbInit( )
{
    //
    // Configure the required pins for USB operation.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOD);
    GPIOPinTypeUSBAnalog(GPIO_PORTD_BASE, GPIO_PIN_4 | GPIO_PIN_5);

    // Erratum workaround for silicon revision A1.  VBUS must have pull-down.
    if(CLASS_IS_TM4C123 && REVISION_IS_A1)
        HWREG(GPIO_PORTB_BASE + GPIO_O_PDR) |= GPIO_PIN_1;

    // Initialize the USB stack for device mode.
    USBStackModeSet( 0, eUSBModeForceDevice, 0 );

    // Set up descriptors and put us on the bus
    USBDCDInit( 0, &lxiDevInfo, (void *)NULL );
}





char XX='0';
static void HandleRequest(void *pvInstance, tUSBRequest *psUSBRequest)
{
    memcpy( &setupReq, psUSBRequest, sizeof( setupReq ) );
    gotUsbRequest   = true;
    gotEP0Data      = false;
    // When LxiUsbPoll is called, it will determine if
    // command is valid, and if so, request the data.
    if (setupReq.bRequest == 0xed) XX = '0';
    UARTCharPut(UART0_BASE,'~');  UARTCharPut(UART0_BASE,++XX);
    DEBUGOUT("[TRVIL=%X,%X,%X,%X,%X,G=%d]\n", setupReq.bmRequestType, setupReq.bRequest, setupReq.wValue, setupReq.wIndex, setupReq.wLength, gotUsbRequest == true);
}



static void HandleEP0DataReceived(void *pvInstance, uint32_t ui32Info)
{
    UARTCharPut(UART0_BASE,'^');  UARTCharPut(UART0_BASE,XX);
    gotEP0Data      = true;
    gotUsbRequest   = true;     // Handle command again, but this time with data.
    LxiUsbPoll();               // This must be in interrupt context.  The USB library completes the transaction upon return.
}



void LxiUsbPoll( )
{
    if( !gotUsbRequest)
        return;

    UARTCharPut(UART0_BASE,'@');  UARTCharPut(UART0_BASE,XX);
    gotUsbRequest = false;

    bool    success     = false;
    bool    read        = (setupReq.bmRequestType & USB_RTYPE_DIR_IN) != 0;
    bool    needData    = false;
    setupReply.wLength  = 0;        // Default to no data returned unless otherwise told to.

    if (read)       // IN, reading
    {
#if 1   //dk9      Disable or remove this at some point.
        if( setupReq.bRequest == TEST_SUCCESS)
        {
            DEBUGOUT(GREEN "TEST_SUCCESS, IN, w/Data:%d\n" RESETATTR, setupReq.wLength);
            // Data and no data
            if (setupReq.wLength) {
                setupReply.wLength = setupReq.wLength;
                success = true;    // It's IN, so send data.

            } else {                    // Couldn't get this to work.  Just don't do IN with no data.
                DEBUGOUT(RED "IN with no data stage.  Couldn't get this to work.  Don't do it.\n"RESETATTR);
                setupReply.wLength = setupReq.wLength;
                success = true;    // It's IN, so send data, even if it's zero bytes.
            }
        } else
        if( setupReq.bRequest == TEST_FAIL)
        {
            DEBUGOUT(GREEN "TEST_FAIL, IN, w/Data:%d\n" RESETATTR, setupReq.wLength);
            // Data and no data
            if (setupReq.wLength) {
                needData =false;
                success = false;    // works to fail transaction
            } else {
                DEBUGOUT(RED "IN with no data stage.  Couldn't get this to work.  Don't do it.\n"RESETATTR);
                needData =false;
                success = true;     // Does this work?  No, doesn't stall.
                success = false;    // Does this stall?  No.
            }
        } else
#endif  //dk9      Disable or remove this at some point.

        //*******************************************************************
        if( setupReq.bRequest == SENSOR_GET_FIRMWARE_VERSION )
        {
            DEBUGOUT(GREEN "SENSOR_GET_FIRMWARE_VERSION\n" RESETATTR);
            xferDataBuf[0] = 1; // MAJOR_VERSION;
            xferDataBuf[1] = 2; // MINOR_VERSION;
            strcpy( (char *)&xferDataBuf[2], "USB Problem Demonstration " );
            strcat( (char *)&xferDataBuf[2], __DATE__ );
            strcat( (char *)&xferDataBuf[2], " " );
            strcat( (char *)&xferDataBuf[2], __TIME__ );
            setupReply.wLength = strlen( (char *)&xferDataBuf[2] ) + 3;
            success = true;
        }
        //*******************************************************************
        // If we want to read data from the processor's address space
        else if( setupReq.bRequest == SENSOR_LOAD_FIRMWARE )
        {
            uint32 addr = ((uint32)setupReq.wIndex << 16) + setupReq.wValue;
            DEBUGOUT(GREEN "SENSOR_LOAD_FIRMWARE (read addr $%X)\n" RESETATTR, addr);
            // Restrict memory reads to valid flash and ram addresses.
            if ( (addr >= TIVA_RAM_BASE) && (addr <TIVA_RAM_END1) )            {
                DEBUGOUT("READ addr %x OK\n", addr);
                if( setupReq.wLength > sizeof( xferDataBuf ) )
                    setupReq.wLength = sizeof( xferDataBuf );
                memcpy( xferDataBuf, (void *)addr, setupReq.wLength );
                setupReply.wLength = setupReq.wLength;
                success=true;
            }
            else
            {
                DEBUGOUT("READ addr %x isn't valid. Rejected.\n", addr);
                success = false;
            }
        }
        //*******************************************************************
        else
            success = false;
    }
    else    // OUT, Writing, not reading.
    {
#if 1   //dk9      Disable or remove this at some point.
        if( setupReq.bRequest == TEST_SUCCESS)
        {
            DEBUGOUT(GREEN "TEST_SUCCESS, OUT, w/Data:%d\n" RESETATTR, setupReq.wLength);
            if (setupReq.wLength) {
                // With data
                if (!gotEP0Data) {
                    needData = true;    // It's OUT, so request data, even if it's zero bytes.
                } else {
                    DEBUGOUT("Got data\n");
                    return;
                }
            } else {
                MAP_USBDevEndpointDataAck( USB0_BASE, USB_EP_0, true );
                return;
            }
        } else
        if( setupReq.bRequest == TEST_FAIL)
        {   // With and without data
            DEBUGOUT(GREEN "TEST_FAIL, OUT, w/Data:%d\n" RESETATTR, setupReq.wLength);
            if (setupReq.wLength)
                success = false;
            else {
                success = false;
            }
        } else
#endif  //dk9      Disable or remove this at some point.

        //*******************************************************************
        // If we want to write data to the processor's address space
        if( setupReq.bRequest == SENSOR_LOAD_FIRMWARE )
        {
            uint32 addr = ((uint32)setupReq.wIndex << 16) + setupReq.wValue;
            DEBUGOUT(GREEN "SENSOR_LOAD_FIRMWARE (write addr $%X)\n" RESETATTR, addr);

            //  if addr is ram
            //      get data
            //      write it to ram
            //  else fail, Write address is bad. Rejected.

            if (addr >= TIVA_RAM_BASE && addr < TIVA_RAM_END1)
            {
                DEBUGOUT("WRITE RAM addr %x is OK\n", addr);
                if (!gotEP0Data)
                {
                    needData = true;
                    setupReq.wLength = min(setupReq.wLength, sizeof(xferDataBuf));
                }
                else
                {
                    memcpy((void*)addr, xferDataBuf, setupReq.wLength);
                    DEBUGOUT("Wrote %d bytes to %x\n", setupReq.wLength, addr);
                    setupReply.wLength = 0;
                    //NO success = true;
                    return; // Made no difference.  Same as success=true.
                    //USBDCDSendDataEP0( 0, 0, 0 );   // Send 0 bytes.
                    //return;
                }
            }
            else
            {
                DEBUGOUT("WRITE addr %x is bad. Rejected.\n", addr);
                success = false;    // Addr is not in RAM.
            }
        }
        //*******************************************************************
        else
            success = false;
    }

   //*******************************************************************
    UARTCharPut(UART0_BASE,'#'); UARTCharPut(UART0_BASE,XX);
    if (needData)
    {
        UARTCharPut(UART0_BASE,'A');
        // Request the data that is being sent to us
        DEBUGOUT(BLUE "Req'ing %d bytes\n" RESETATTR, setupReq.wLength);
        USBDCDRequestDataEP0( 0, xferDataBuf, setupReq.wLength );
        // This allows the data to start coming in.
        MAP_USBDevEndpointDataAck( USB0_BASE, USB_EP_0, false );
    } else if (success)
    {
        UARTCharPut(UART0_BASE,'B');
        DEBUGOUT(BLUE "Acking with %d bytes\n" RESETATTR, setupReply.wLength);
        USBDCDSendDataEP0( 0, xferDataBuf, setupReply.wLength );
    }
    else
    {
        UARTCharPut(UART0_BASE,'C');
        DEBUGOUT(BLUE "STALL--Req:%02X  Val:%02X Idx:%02X Len:%02X Read:%d\n" RESETATTR,
         setupReq.bRequest, setupReq.wValue, setupReq.wIndex, setupReq.wLength, read);
        USBDCDStallEP0( 0 );
    }
}
