/*
 * LxiUsbDescriptors.c
 *
 * The  USB descriptors.
 *
 */

#include <stdbool.h>
#include <stdint.h>
#include "driverlib/usb.h"
#include "usblib/usblib.h"
#include "usblib/usbhid.h"
#include "usblib/device/usbdevice.h"
#include "usblib/device/usbdhid.h"

extern const tCustomHandlers lxiUsbHandlers;        // In LxiUsb.c



//*****************************************************************************
static uint8_t lxiDeviceDescriptor[] =
{
    18,                         // Size of this structure.
    USB_DTYPE_DEVICE,           // Type of this structure.
    USBShort(0x110),            // USB version 1.1 (if we say 2.0, hosts assume
                                // high-speed - see USB 2.0 spec 9.2.6.6)
    0,                          // USB Device Class
    0,                          // USB Device Sub-class
    0,                          // USB Device protocol
    64,                         // Maximum packet size for default pipe.
    USBShort(0x0945),           // Vendor ID (VID).
    USBShort(0x0002),           // Product ID (PID).
    USBShort(0x100),            // Device Version BCD.
    1,                          // Manufacturer string identifier.
    2,                          // Product string identifier.
    0,                          // Product serial number.
    1                           // Number of configurations.
};


// Configuration descriptor header.
static uint8_t lxiConfigDescriptor[] =
{
    9,                          // Size of the configuration descriptor.
    USB_DTYPE_CONFIGURATION,    // Type of this descriptor.
    USBShort(25),               // The total size of this full structure.
    1,                          // The number of interfaces in this configuration.
    1,                          // The unique value for this configuration.
    0,                          // The string identifier that describes this configuration.
    USB_CONF_ATTR_BUS_PWR,     // Bus Powered, Self Powered, remote wake up.
    250,                        // The maximum power in 2mA increments.
};


//*****************************************************************************
static uint8_t lxiInterfaceDescriptor[HIDINTERFACE_SIZE] =
{
    9,                          // Size of the interface descriptor.
    USB_DTYPE_INTERFACE,        // Type of this descriptor.
    0,                          // The index for this interface.
    0,                          // The alternate setting for this interface.
    1,                          // The number of endpoints used by this interface.
    USB_CLASS_VEND_SPECIFIC,    // The interface class
    0,                          // The interface sub-class.
    0,                          // The interface protocol for the sub-class specified above.
    0,                          // The string index for this interface.
};


// Define EP2In bulk endpoint
static const uint8_t lxiInEndpointDescriptor[] =
{
    7,                          // The size of the endpoint descriptor.
    USB_DTYPE_ENDPOINT,         // Descriptor type is an endpoint.
    USB_EP_DESC_IN | 2,         // EP2IN
    USB_EP_ATTR_BULK,           // Endpoint is a bulk endpoint.
    USBShort( 64 ),             // The maximum packet size.
    255                         // The polling interval for this endpoint.
};



static const tConfigSection lxiConfigSection =
{
    sizeof(lxiConfigDescriptor),
    lxiConfigDescriptor
};

static const tConfigSection lxiInterfaceSection =
{
    sizeof(lxiInterfaceDescriptor),
    lxiInterfaceDescriptor
};

static const tConfigSection lxiInEndpointSection =
{
    sizeof(lxiInEndpointDescriptor),
    lxiInEndpointDescriptor
};


//*****************************************************************************
static const tConfigSection *lxiSections[] =
{
    &lxiConfigSection,
    &lxiInterfaceSection,
    &lxiInEndpointSection
};

#define NUM_LXI_SECTIONS        (sizeof(lxiSections) / sizeof(lxiSections[0]))

//*****************************************************************************
static tConfigHeader lxiConfigHeader =
{
    NUM_LXI_SECTIONS,
    lxiSections
};

//*****************************************************************************
static const tConfigHeader * const lxiConfigDescriptors[] =
{
    &lxiConfigHeader
};

//*****************************************************************************
const uint8_t lxiManufacturerString[] =
{
    (14 + 1) * 2,
    USB_DTYPE_STRING,
    'N', 0, 'o', 0, 'b', 0, 'o', 0, 'd', 0, 'y', 0, ' ', 0, 'S', 0, 'p', 0,
    'e', 0, 'c', 0, 'i', 0, 'a', 0, 'l', 0
};

//*****************************************************************************
const uint8_t lxiProductString[] =
{
    (10 + 1) * 2,
    USB_DTYPE_STRING,
    'U', 0, 'S', 0, 'B', 0, 'p', 0, 'r', 0, 'o', 0, 'b', 0, 'l', 0, 'e', 0, 'm', 0
};

//*****************************************************************************
const uint8_t lxiLangDescriptor[] =
{
    4,
    USB_DTYPE_STRING,
    USBShort(USB_LANG_EN_US)
};


//*****************************************************************************
const uint8_t *const lxiStringDescriptors[] =
{
    lxiLangDescriptor,
    lxiManufacturerString,
    lxiProductString
};

#define numLxiStringDescriptors (sizeof( lxiStringDescriptors ) / sizeof( lxiStringDescriptors[0] ))

//*****************************************************************************
// Basic USB descriptor info and callbacks for the LXi
//*****************************************************************************
tDeviceInfo lxiDevInfo =
{
    &lxiUsbHandlers,
    lxiDeviceDescriptor,
    lxiConfigDescriptors,
    lxiStringDescriptors,
    numLxiStringDescriptors
};


