/*
 * console.h
 *
 *  Created on: Feb 8, 2022
 *      Author: dkaden
 *
 * This is intended to be used for conditional and unconditional UART output.
 * Unconditional output is used when displaying test results.  Conditional
 * output is used when displaying debug output.  ANSI color codes are also
 * defined for color output.
 *
 * To use conditional output, use these macros.  When compiled in release mode,
 * they evaluate to nothing.
 * 		DEBUGINIT
 * 		DEBUGOUT(...)
 *
 * 	For unconditional output use these functions:
 * 		InitConsole();
 * 		UARTprintf(...);
 * 		The macro CONOUT(...) can also be used.  (Evaluates to UARTprintf.)
 */

#ifndef CONSOLE_H_
#define CONSOLE_H_

#include <stdint.h>		// uartstdio.h doesn't include headers it refers to.  Do that here.
#include <stdbool.h>
#include <uartstdio.h>
#include "driverlib/interrupt.h"


enum
{
    ESC     = 0x1B      // ESC key on keyboard.
};


#ifdef DEBUG
void	debugout(const char *msg, ...);

#define DEBUGINIT			InitConsole()
#define DEBUGOUT(...)		do { IntMasterDisable(); debugout(__VA_ARGS__); IntMasterEnable(); } while (0)

void UartHexOutB(uint8_t  val);
void UartHexOutW(uint16_t val);

#else

#define DEBUGINIT			do { } while (0)
#define DEBUGOUT(...)       do { } while (0)
#define UartHexOutB(...)    do { } while (0)
#define UartHexOutW(...)    do { } while (0)

#endif

#ifdef UART_BUFFERED
    #define	PAUSE	do { while(!UARTRxBytesAvail()); UARTFlushRx(); } while(0)
    #define FLUSHUARTTX()     UARTFlushTx(false)
#else
    #define PAUSE   do { UARTgetc(); } while(0)
    #define FLUSHUARTTX()
#endif

void		InitConsole(void);

// Use UARTprintf() for unconditional output.
#define		CONOUT		UARTprintf


// ==========================  ANSI COLOR CODES  =============================
// Define some ANSI terminal control codes.  These work in the CCS debug terminal.
// and UART terminal output (if terminal program supports them).

	#define RESETATTR	"\x1B[0m"		// Reset all attributes
	#define BRIGHT		"\x1B[1m"		// Bright.
	#define UNDERSCORE	"\x1B[4m"		// Underscore

	// Foreground colors.
	#define BLACK		"\x1B[30m"
	#define RED			"\x1B[31m"
	#define GREEN		"\x1B[32m"
	#define YELLOW		"\x1B[33m"
	#define BLUE		"\x1B[34m"
	#define MAGENTA		"\x1B[35m"
	#define CYAN		"\x1B[36m"
	#define WHITE		"\x1B[37m"

	// Background colors.
	#define BGBLACK		"\x1B[40m"
	#define BGRED		"\x1B[41m"
	#define BGGREEN		"\x1B[42m"
	#define BGYELLOW	"\x1B[43m"
	#define BGBLUE		"\x1B[44m"
	#define BGMAGENTA	"\x1B[45m"
	#define BGCYAN		"\x1B[46m"
	#define BGWHITE		"\x1B[47m"

#endif /* CONSOLE_H_ */
